package xyz.pixelatedw.finalbeta;

import java.util.ArrayList;
import java.util.List;

import com.electronwill.nightconfig.core.file.CommentedFileConfig;
import com.electronwill.nightconfig.core.file.CommentedFileConfigBuilder;
import com.google.common.base.Strings;

public class ModConfig {

	private static final String CONFIG_PATH = "config/finalbeta.toml";
	private static final List<Option> OPTIONS = new ArrayList<>();

	private CommentedFileConfig fileConfig;

	public static final Option<Boolean> SUGAR_CANE_ON_SAND = make("Sugar Cane on sand", true, "Allows sugar canes to be placed on sand");
	public static final Option<Boolean> ADD_MORE_SOUNDS = make("Add more sounds", true,
			"Links a few more sounds from your local 'resources' folder with the game, namely for item breaking, minecarts and chests");
	public static final Option<Boolean> FIX_BOW_MODEL = make("Fix bow model", true,
			"Makes the box model held by players and skeletons bigger and facing forward");
	public static final Option<Boolean> FIX_MINECART_FLICKERING = make("Fix minecart flickering", true,
			"Fixes minecarts flickering when looking at their backs as a passanger");
	public static final Option<Boolean> FIX_MINECART_STOPPING_ON_ITEMS = make("Fix minecart stopping on items", true,
			"Fixes minecarts getting stopped by arrows and dropped items on tracks");
	public static final Option<Boolean> FIX_FISHING = make("Fix fishing", true, "Fixes fishes going above the player's head when fishing");
	public static final Option<Boolean> FIX_LEG_ARMOR_ON_VEHICLES = make("Fix leg armor on vehicles", true,
			"Fixes leg armor not properly getting updated when switching poses (start/stop riding a vehicle)");
	public static final Option<Boolean> FIX_STAIRS_DROPS = make("Fix stairs drops", true, "Fix stairs not dropping themselves when mined");
	public static final Option<Boolean> FIX_PICKAXE_EFFECTIVENESS = make("Fix pickaxe effectiveness", true,
			"Fixes pickaxes not being effective agaist certain blocks that it should be effective on");
	public static final Option<Boolean> FIX_AXE_EFFECTIVENESS = make("Fix axe effectiveness", true,
			"Fixes axes not being effective agaist certain blocks that it should be effective on");
	public static final Option<Boolean> FIX_SADDLES_NOT_DROPPING = make("Fix saddles not dropping", true,
			"Fixes saddles not dropping when killing saddled pigs");
	
	private static ModConfig instance = new ModConfig();
	public static final ModConfig instance() {
		return instance;
	}

	public ModConfig() {
		CommentedFileConfigBuilder builder = CommentedFileConfig.builder(CONFIG_PATH);
		this.fileConfig = builder.autosave().autoreload().build();

		this.fileConfig.load();

		boolean save = false;
		for (Option o : OPTIONS) {
			if (!this.fileConfig.contains(o.name)) {
				this.fileConfig.add(o.name, o.defaultValue);
				this.fileConfig.setComment(o.name, o.description);
				save = true;
			} else if (!Strings.isNullOrEmpty(o.description) && this.fileConfig.contains(o.name)
					&& !this.fileConfig.containsComment(o.name)) {
				this.fileConfig.setComment(o.name, o.description);
				save = true;
			}
		}

		if (save) {
			this.fileConfig.load();
		}
	}

	public static <T> Option<T> make(String name, T defaultValue, String description) {
		Option<T> option = new Option(name, defaultValue, description);
		OPTIONS.add(option);
		return option;
	}

	public <T> T get(Option<T> o) {
		return this.fileConfig.getOrElse(o.name, o.defaultValue);
	}

	public static class Option<T> {
		private String name;
		private String description;
		private T defaultValue;

		public Option(String name, T defaultValue, String description) {
			this.name = name;
			this.defaultValue = defaultValue;
			this.description = description;
		}

		public Option(String name, T defaultValue) {
			this.name = name;
			this.defaultValue = defaultValue;
		}

		public T get() {
			return ModConfig.instance().get(this);
		}
	}
}
